function [Population] = EnvironmentSelectionDecomposition(Population,W)
% The environmental selection of the decomposition-based method.

[Nr,m] = size(W);
[np,n] = size(Population);
f = Population(:,n-m+1:n);
Z = zeros(1,m);

%% Translate the population
f = (f-repmat(min(f),np,1))./(repmat(max(f),np,1)-repmat(min(f),np,1));

%% Associate each solution to a reference vector
Cosinetemp = pdist2(f,W,'cosine');
Cosine = 1-Cosinetemp;
[Cosinemax,associate] = max(Cosine,[],2);
Next = zeros(Nr,1);

%% Select one solution for each reference vector
list = unique(associate)';
for i = list
    current = find(associate == i);
    d = pdist2(f(current,:),repmat(Z,length(current),1),'Euclidean');
    Fan = Cosinemax(current)./d(:,1);
    [~,best] = max(Fan);
    Next(i)  = current(best);
end
% Population for next generation
Population = Population(Next(Next~=0),:);

end
